﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace Diagnostic
{
    /// <summary>
    /// ImageManager slouzi k ukladani snimku v prubehu sledovani, je schopen ukladat puvodni snimek a snimek zpetne projekce
    /// spilecne se sledovacimi oblastmi. Tato trida je ciste diagnostickeho charakteru a byla vytvorena za ucely extrakce snimku
    /// pro diplomovou praci. V pripade potřeby užiti je potreba prejmenovat constantu PATH  a namapovat cestu vlastniho adresare.
    /// </summary>
    public static class ImageManager
    {
        private const string PATH = "C:\\CameraStore\\Picture sets\\";
        private const string EXTENSION = ".jpg";
        private const string BACKPROJECTION = "bp";
        private static int _imageCoutner;
        private static string _folderPath;
        private static Pen _penTarget = new Pen(Color.Green);

        public static void NewFolderRequest()
        {
            string[] directories = Directory.GetDirectories(PATH);
            _folderPath = System.IO.Path.Combine(PATH, (directories.Length + 1).ToString());
            Directory.CreateDirectory(_folderPath);
            _imageCoutner = 0;

        }
        public static void SaveImagesSet(Bitmap trackingW, Bitmap backProjection, Size size, TrackerGraphic.TargetArea area)
        {
            _imageCoutner++;
            if (area != null && area.ObjectArea != null) {

                using (Graphics g = Graphics.FromImage(trackingW as Image)) {
                    g.DrawRectangle(_penTarget, area.ObjectArea.AreaRectangle);
                }
            }
            if (!size.IsEmpty) {
                trackingW = (Bitmap)trackingW.GetThumbnailImage(size.Width, size.Height, null, IntPtr.Zero);
                if (backProjection != null)
                    backProjection = (Bitmap)backProjection.GetThumbnailImage(size.Width,
                        size.Height, null, IntPtr.Zero);
            }
            if (null != trackingW)
                trackingW.Save(_folderPath + "\\" + GenerateLabel(false, _imageCoutner), ImageFormat.Jpeg);
            if (null != backProjection)
                backProjection.Save(_folderPath + "\\" + GenerateLabel(true, _imageCoutner));
        }
        private static string GenerateLabel(bool backProjection, int count)
        {
            string s = count.ToString();
            s = backProjection ? s + BACKPROJECTION : s;
            return s + EXTENSION;
        }
    }
}
