﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;
using TrackerGraphic;

namespace Diagnostic
{
    public class TrackerIterationDiagnostic
    {
        private const string PATH = "C:\\CameraStore\\Diagnostics\\";
        private const string EXTENSION = ".jpg";
        private List<Bitmap> _framesList;
        private List<String> _names;
        private Pen _targetPen;
        private int _frameCounter;
        private int _timerCheckCounter;
        private Timer _checkTimer;

        public TrackerIterationDiagnostic()
        {
            this._framesList = new List<Bitmap>();
            this._names = new List<string>();
            this._checkTimer = new Timer();
            this._checkTimer.Interval = 2000;
            this._targetPen = new Pen(Color.GreenYellow, 2);
            this._checkTimer.Tick += new EventHandler(m_checkTimer_Tick);
        }

        void m_checkTimer_Tick(object sender, EventArgs e)
        {
            if (this._frameCounter > this._timerCheckCounter) {
                this._timerCheckCounter = this._frameCounter;
            } else {
                this.SaveResults();
                this._checkTimer.Stop();
            }
        }
        private Bitmap MarkTarget(Bitmap frame, TargetArea area)
        {
            Graphics g = Graphics.FromImage((Image)frame);

            g.DrawRectangle(this._targetPen, area.ObjectArea.AreaRectangle);

            return frame;
        }
        private string GetFrameName(int no, int iteration)
        {
            string s = String.Format("frame {0} iter {1}", no, iteration);
            return s;
        }
        public void Add(Bitmap frame, TargetArea area, int iteration)
        {

            if (iteration == 0) {
                this._frameCounter++;
            }
            frame = this.MarkTarget(frame, area);
            this._framesList.Add(frame);
            this._names.Add(GetFrameName(this._frameCounter, iteration));
            if (this._frameCounter > 1)
                this._checkTimer.Start();
        }
        private void SaveResults()
        {
            //string time = DateTime.Now.ToLongTimeString();
            string[] directories = Directory.GetDirectories(PATH);
            string newPath = System.IO.Path.Combine(PATH, (directories.Length + 1).ToString());
            Directory.CreateDirectory(newPath);
            for (int i = 0; i < this._framesList.Count - 1; i++) {
                //string s = newPath + "\\" + i.ToString() + EXTENSION;
                string s = newPath + "\\" + _names[i].ToString() + EXTENSION;
                Bitmap bmp = _framesList[i];
                //bmp = Resize(bmp);
                bmp.Save(s, ImageFormat.Jpeg);
            }
        }
        public static Bitmap Resize(Bitmap bmp)
        {
            Bitmap temp = new Bitmap(640, 480);
            Graphics g = Graphics.FromImage(temp);
            g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
            g.DrawImage(bmp, 0, 0, temp.Width, temp.Height);
            bmp.Dispose();
            g.Dispose();

            return temp;
            ;
        }

    }
}
