﻿using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace RGBHistogramGenerator
{
    /// <summary>
    /// Staticka nechranena metoda umoynuje ziskavat obraz zpetne projekce k jednotlivym snimkum
    /// </summary>
    public static unsafe class BackProjection
    {
        private static Bitmap _backProjection;
        private static BitmapData _data;
        private static BitmapData _dataBP;
        private static unsafe int* posBP;
        private static unsafe int* pos;
        private static int aInt = -16777216;
        private static byte r, g, b;


        /// <summary>
        /// Ziska obraz zpetne projekce predaneho s nimku, k pomeru histogramu cile a histogramu celeho obrazku
        /// </summary>
        /// <param name="bmp">bitmapa ke zpracovani</param>
        /// <param name="targetHistogram">Histogram cile.</param>
        /// <param name="wholeImageHistogram">Histogram prohledavaci oblasti - celeho snimku</param>
        /// <returns></returns>
        public static Bitmap GetBackProjectionImage(ref Bitmap bmp, BaseHistogram targetHistogram, BaseHistogram wholeImageHistogram)
        {
            //pripad 1: Pouziti stejneho kernelu jak pro popredi tak pozadi u BGW - stary zpusob
            //pripad 2: 2 odlisne kernely

            // pripad 2
            //Single temp = byte.MaxValue / targetHistogram.GetLargestEntry();
            Single _targetFrequency = 0;
            Single _imageFrequency = 0;
            byte _clrVal = byte.MinValue;
            _backProjection = new Bitmap(bmp.Width, bmp.Height);
            _data = bmp.LockBits(
                new Rectangle(0, 0, bmp.Width, bmp.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            _dataBP = _backProjection.LockBits(
                new Rectangle(0, 0, _backProjection.Width, _backProjection.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            for (int height = 0; height < _data.Height; height++) {
                posBP = (int*)((int)_dataBP.Scan0 + (height * _dataBP.Stride));
                pos = (int*)((int)_data.Scan0 + (height * _data.Stride));
                for (int width = 0; width < _data.Width; width++) {
                    BackProjection.IntArgbToByteValues(pos, out r, out g, out b);
                    _targetFrequency = targetHistogram.GetColorFrequency(r, g, b);
                    _imageFrequency = wholeImageHistogram.GetColorFrequency(r, g, b);

                    //pripad 2
                    //_clrVal = Math.Min((byte)((temp * (_targetFrequency))), byte.MaxValue);
                    //pripad 1
                    _clrVal = (byte)((_imageFrequency > 0) ? (int)(_targetFrequency / _imageFrequency) * byte.MaxValue : 0);    //urychleni + 1ms
                    *posBP = BackProjection.ByteValuesToIntArgb(_clrVal, _clrVal, _clrVal);
                    pos++;
                    posBP++;
                }
            }
            bmp.UnlockBits(_data);
            _backProjection.UnlockBits(_dataBP);
            return _backProjection;
        }

        /// <summary>
        /// Extrahuje z barevne hodnoty ARGB ulozene v pameti,  jednotlive barevne slozky v bytech 
        /// </summary>
        /// <param name="pos">The pos.</param>
        /// <param name="red">The red.</param>
        /// <param name="green">The green.</param>
        /// <param name="blue">The blue.</param>
        public static unsafe void IntArgbToByteValues(int* pos, out byte red, out byte green, out byte blue)
        {
            blue = (byte)(*pos & 0xFF);
            green = (byte)((*pos >> 8) & 0xFF);
            red = (byte)((*pos >> 16) & 0xFF);
        }
        /// <summary>
        /// Prevede byty jednotlivych barevnych slozek na hodnotu ARGB , ktera se uklada do pameti.
        /// </summary>
        /// <param name="red">The red.</param>
        /// <param name="green">The green.</param>
        /// <param name="blue">The blue.</param>
        /// <returns></returns>
        public static int ByteValuesToIntArgb(byte red, byte green, byte blue)
        {
            return aInt + (red << 16) + (green << 8) + (blue << 0);
        }
        static unsafe int ByteValuesToIntArgb(byte a, byte r, byte g, byte b)
        {
            return (a << 24) + (r << 16) + (g << 8) + (b << 0);
        }

    }
}
