﻿using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace RGBHistogramGenerator
{
    public class HistogramKernelBased : HistogramUnweighted
    {
        private Point _center;
        private Single _distance;
        private Single _kernelValue;
        private Single _pixels;
        private Single _Xn;
        private Single _Yn;
        private BitmapData _data;
        /// <summary>
        /// Initializes a new instance of the <see cref="HistogramKernelBased"/> class.
        /// </summary>
        /// <param name="red">The red.</param>
        /// <param name="green">The green.</param>
        /// <param name="blue">The blue.</param>
        public HistogramKernelBased(int red, int green, int blue)
            : base(red, green, blue)
        {
        }
        /// <summary>
        /// Vypočítá histogram z předané bitmapy
        /// </summary>
        /// <param name="bmp">bitmapa z ktere ma byt vypocitan histogram</param>
        /// <returns>vygenerovaný histogram</returns>
        public override unsafe BaseHistogram CalculateHistogram(ref System.Drawing.Bitmap bmp)
        {
            _center = new Point(bmp.Width / 2, bmp.Height / 2);
            _distance = 0;
            _kernelValue = 0;
            _pixels = 0;
            _Xn = 0;
            _Yn = 0;
            byte r, g, b;
            _data = bmp.LockBits(
                new Rectangle(0, 0, bmp.Width, bmp.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            int* pos;
            for (int y = 0; y < _data.Height; y++) {
                pos = (int*)(_data.Scan0.ToInt32() + (y * _data.Stride));
                for (int x = 0; x < _data.Width; x++) {
                    _Xn = (Single)2 * (x - _center.X) / bmp.Width;
                    _Yn = (Single)2 * (y - _center.Y) / bmp.Height;
                    _distance = (Single)Math.Sqrt(_Xn * _Xn + _Yn * _Yn);

                    if (_distance * _distance < 1) {
                        _kernelValue = (Single)(2 * ((1 - _distance) / Math.PI));
                    } else
                        _kernelValue = 0;
                    BackProjection.IntArgbToByteValues(pos, out r, out g, out b);
                    base.AddColorValue(r, g, b, _kernelValue);
                    pos++;
                    _pixels++;
                }
            }
            bmp.UnlockBits(_data);
            this.NormaliseHistogram(_pixels);
            return this;
        }
    }
}
