﻿using System;

namespace RGBHistogramGenerator
{
    public static class KernelProfilesLibrary
    {
        private static readonly int BACKGROUND_BANDWIDTH = 3;
        private static readonly int BACKGROUND_CONSTANT = 1;

        public static Single DefaultForegroundProfile(Single radius)
        {
            return TriangularKernel(radius);
            //return radius <= 1 ? 1 - radius : 0;
        }

        public static Single DefaultBackGroundProfile(Single radius)
        {
            if (1 < radius && radius <= BACKGROUND_BANDWIDTH) {
                return BACKGROUND_CONSTANT * radius;
            } else {
                return 0;
            }
        }

        public static Single EpanchikovKernel(Single distance)
        {
            return distance <= 1 ? (Single)(2 * ((1 - distance * distance) / Math.PI)) : 0;

            //return distance <= 1 ? (Single)((3/4) * ((1 - distance * distance))) : 0;
        }

        public static Single GausianKernel(Single distance)
        {
            Single temp = (-0.5f) * distance * distance;
            return (Single)(1 / Math.Sqrt(2 * Math.PI)) * (Single)Math.Pow(Math.E, temp);
        }
        public static Single QuarticKernel(Single distance)
        {
            if (distance <= 1) {
                Single temp = (1 - distance * distance);
                return (15f / 16f) * ((Single)Math.Pow(temp, 2));
            } else {
                return 0;
            }
        }

        public static Single Triweight(Single distance)
        {
            if (distance <= 1) {
                Single temp = (1 - distance * distance);
                return (35f / 32f) * (Single)Math.Pow(temp, 3);
            } else {
                return 0;
            }
        }

        public static Single TriangularKernel(Single distance)
        {
            distance = Math.Abs(distance);
            if (distance <= 1) {
                return (1 - distance);
            } else {
                return 0;
            }
        }
        public static Single CosineKernel(Single distance)
        {
            if (distance <= 1) {
                return (Single)(Math.PI / 4f) * (Single)Math.Cos((Math.PI / 2) * distance);
            } else
                return 0;
        }

    }
}
