﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace UI
{
    public delegate void WindowClosed(bool isClosed);

    public partial class BackProjectionForm : Form
    {
        #region constants
        private const int DEFAULT_PADDING = 40;
        #endregion

        public event WindowClosed OnWindowClosed;
        private static BackProjectionForm _instance;

        public static BackProjectionForm Instance
        {
            get
            {
                if (_instance == null) {
                    _instance = new BackProjectionForm();
                }
                return BackProjectionForm._instance;
            }
            private set
            {
                BackProjectionForm._instance = value;
            }
        }

        Form _parent;
        public bool IsLoaded
        {
            get;
            private set;
        }
        private bool IsForcedClosing
        {
            get;
            set;
        }
        private BackProjectionForm()
        {
            InitializeComponent();
        }

        public static void SetParent(Form parent)
        {
            Instance.MaximizeBox = false;
            Instance._parent = parent;
            Instance.StartPosition = FormStartPosition.Manual;
            Instance.IsForcedClosing = false;
            Instance.Visible = false;
        }

        public void SetImage(Bitmap bmp)
        {
            this.picutureBoxBP.Image = bmp;
        }
        public void DockWindow()
        {
            this.Location = new Point(this._parent.Right, this._parent.Top);

        }
        public void ForcedClose()
        {
            this.IsForcedClosing = true;
            this.Close();
        }

        #region Overriden methos
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.IsLoaded = true;
        }
        protected override void OnClosing(CancelEventArgs e)
        {
            e.Cancel = !this.IsForcedClosing;
            base.OnClosing(e);
            this.Visible = false;
            this.ShowInTaskbar = false;
            if (OnWindowClosed != null) {
                OnWindowClosed(e.Cancel);
            }
        }
        protected override void OnActivated(EventArgs e)
        {
            base.OnActivated(e);
            this.Visible = true;
            this.ShowInTaskbar = true;
        }
        #endregion





    }
}
