﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace UI
{
    public partial class RecordForm : Form
    {
        private const string START = "Start";
        private const string STOP = "Stop";
        private const string FILENAME_EXTENSION = ".jpg";


        private FrameCapturer _frameCapturer;
        private string _path;
        private int _counter;
        private bool _isRunning;

        public RecordForm()
        {
            InitializeComponent();

            _frameCapturer = new FrameCapturer();
            this.btnStart.Enabled = false;
            #region Event Handlers
            this.btnStart.Click += new EventHandler(btnStart_Click);
            _frameCapturer.DeviceFound += new EventHandler(_frameCapturer_DeviceFound);
            _frameCapturer.FrameCaptured += new FrameCapturer.FrameCaptureEventHandler(_frameCapturer_FrameCaptured);

            #endregion
        }

        void _frameCapturer_DeviceFound(object sender, EventArgs e)
        {
            this.btnStart.Enabled = true;

            if (this._frameCapturer.FilterInfoCollection.Count > 0) {
                Thread.Sleep(1000);
                this._frameCapturer.SetDevice(1);
                if (this._frameCapturer.IsDeviceSet) {
                    _frameCapturer.DeviceFound -= new EventHandler(_frameCapturer_DeviceFound);
                    this._frameCapturer.Start();
                    Thread.Sleep(800);
                }
            } else {
                MessageBox.Show("Nerozpoznana zadna kamera");
            }
        }

        void _frameCapturer_FrameCaptured(object sender, Bitmap e)
        {
            try {
                pictureBoxRecord.Invoke(new MethodInvoker(() => {
                    if (this.pictureBoxRecord.Size != e.Size) {
                        this.pictureBoxRecord.Size = new Size(e.Width, e.Height);
                    }
                    this.pictureBoxRecord.Image = e.Clone() as Bitmap;
                }));
            } catch (Exception ex) {
                Debug.WriteLine(ex.Message);
            }
            Debug.WriteLine("CAPTURED");
            if (this._isRunning) {
                e.Save(this._path + "\\" + (this._counter++).ToString() + FILENAME_EXTENSION);
            }
        }

        #region Event Handlers
        void btnStart_Click(object sender, EventArgs e)
        {
            if (this.btnStart.Text == START) {
                this.NewDialogRequest();
            } else {
                Reset();
            }
        }


        #endregion

        #region Methods
        private void NewDialogRequest()
        {
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            DialogResult result = dlg.ShowDialog(this);
            if (result == System.Windows.Forms.DialogResult.OK) {
                this.btnStart.Text = STOP;
                this._path = dlg.SelectedPath;
                this.StartRecording();
            }
        }
        private void Reset()
        {
            this.btnStart.Text = START;
            this._isRunning = false;
            this._path = string.Empty;
            this._counter = 0;
        }
        private void StartRecording()
        {
            if (!String.IsNullOrEmpty(this._path) && Directory.Exists(_path) && this._frameCapturer.IsDeviceSet) {
                this._isRunning = true;
            }
        }
        protected override void OnClosing(CancelEventArgs e)
        {
            this._frameCapturer.Stop();
            _frameCapturer.FrameCaptured -= new FrameCapturer.FrameCaptureEventHandler(_frameCapturer_FrameCaptured);
            base.OnClosing(e);
        }
        #endregion
    }
}
