﻿using System.Drawing;
using RGBHistogramGenerator;
using RGBHistogramGenerator.Enumerations;
using TrackerGraphic;

namespace Tracker
{
    public class CAMShift : CAMShiftDM
    {
        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="ScaleAdaptativeMS"/> class.
        /// </summary>
        /// <param name="numberOfRedBins">The number of red bins.</param>
        /// <param name="numberOfGreenBins">The number of green bins.</param>
        /// <param name="numberOfBlueBins">The number of blue bins.</param>
        /// <param name="iterations">The iterations.</param>
        /// <param name="type">The type.</param>
        /// <param name="bmp">The BMP.</param>
        /// <param name="area">The area.</param>
        public CAMShift(int numberOfRedBins, int numberOfGreenBins, int numberOfBlueBins, int iterations, HistogramType type, ref Bitmap bmp, TargetArea area)
            : base(numberOfRedBins, numberOfGreenBins, numberOfBlueBins, iterations, type, ref bmp, area)
        {
        }
        #endregion //end constructor
        /// <summary>
        /// Scale adaptive Mean shift sledování, po nalezení cíle odešle informace o jeho poloze a informace o vypoctu.
        /// Velikost sledovaciho okna je implicitne prenastavovana
        /// </summary>
        /// <param name="bmp">Bitmapa k analyze</param>
        public override void Tracking(ref Bitmap bmp)
        {
            base.StopWatch.Start();
            Point centroid;
            int iterations = 0;
            base._sample = bmp;
            //base.m_sample = TargetSelection.GetImageSample(ref bmp, base.PreviousArea.SearchingArea);
            base.HistogramGenerator.GenerateHistogram(SourceType.WholeImage, ref base._sample);
            this._backProjection = BackProjection.GetBackProjectionImage(ref base._sample,
                base.HistogramGenerator.Target, base.HistogramGenerator.WholeImage);
            this._searchingAreaLocation = base.PreviousArea.SearchingArea.AreaRectangle.Location;
            base._sample = TargetSelection.GetImageSample(ref this._backProjection, base.PreviousArea, ref bmp);
            for (int i = 0; i < base.MaximumIterations; i++) {
                base._sample = TargetSelection.GetImageSample(ref this._backProjection, _searchingAreaLocation, base.PreviousArea, ref bmp);
                data = DistributionCalculator.GetDistributionInformations(base._sample);
                //if (base.CurrentArea is IAdaptiveArea) {
                //    if (base.IsSimpleSizeCalculation) {
                //        (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.Size);
                //    } else {
                //        (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.DistributionSize);
                //    }
                //}
                centroid = DistributionCalculator.RecalculateCentroidToArea(data.CentroidPosition, base.PreviousArea);

                base.CurrentArea = new AdaptiveTargetArea(base.PreviousArea);
                base.CurrentArea.OffSet(centroid.X, centroid.Y);
                if (TargetArea.CheckBounds(base.CurrentArea, ref bmp))
                    base.CurrentArea = base.PreviousArea;
                if (base.CurrentArea.Center.Equals(base.PreviousArea.Center) || i == base.MaximumIterations) {
                    iterations = ++i;
                    break;
                }
                base.PreviousArea = base.CurrentArea;
            }
            if (base.CurrentArea is IAdaptiveArea) {
                if (base.IsSimpleSizeCalculation) {
                    (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.Size);
                } else {
                    (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.DistributionSize);
                }
            }
            base.PreviousArea = base.CurrentArea;
            this.OnTargetFound(new CAMShiftEventArgs(bmp, this._backProjection, null, base.CurrentArea,
                base.HistogramGenerator.CandidateHistogram, iterations + 1, base.StopWatch.ElapsedMilliseconds, null));
            base.StopWatch.Reset();
        }
    }
}
