﻿using System.Drawing;
using RGBHistogramGenerator;
using RGBHistogramGenerator.Enumerations;
using TrackerGraphic;

namespace Tracker
{
    public class CAMShiftDM : MeanShift
    {
        #region Fields
        protected DistributionData data;
        protected Bitmap _backProjection;
        protected Point _searchingAreaLocation;
        #endregion //end fields

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="CAMShiftDM"/> class.
        /// </summary>
        /// <param name="numberOfRedBins"></param>
        /// <param name="numberOfGreenBins"></param>
        /// <param name="numberOfBlueBins"></param>
        /// <param name="iterations">maximalni pcoet  mean shift iteraci</param>
        /// <param name="type"></param>
        /// <param name="bmp">bitmapa k analyzovani</param>
        /// <param name="area">pozice cíle</param>
        public CAMShiftDM(int numberOfRedBins, int numberOfGreenBins, int numberOfBlueBins, int iterations,
            HistogramType type, ref Bitmap bmp, TargetArea area)
            : base(numberOfRedBins, numberOfGreenBins, numberOfBlueBins, iterations, type, ref bmp, area)
        {
        }
        #endregion //end constructor

        #region Methods
        /// <summary>
        /// CAMShift sledování, po nalezení cíle odešle informace o jeho poloze a informace o vypoctu.
        /// Histogram cile ja implicitne updatovan pro kazdy novy snimek, velikost sledovaciho okna je implicitne
        /// prizpusobovana
        /// </summary>
        /// <param name="bmp">Bitmapa k analyze</param>
        public override void Tracking(ref Bitmap bmp)
        {
            base.StopWatch.Start();
            Point centroid;
            int iterations = 0;
            //bmp = TargetSelection.GetImageSample(ref bmp, base.PreviousArea.SearchingArea);
            //this.m_searchingAreaLocation = base.PreviousArea.SearchingArea.AreaRectangle.Location;

            base.HistogramGenerator.GenerateHistogram(SourceType.WholeImage, ref bmp);
            this._backProjection = BackProjection.GetBackProjectionImage(ref bmp,
                base.HistogramGenerator.Target, base.HistogramGenerator.WholeImage);
            // while (!this._isReady && this._secCounter < 20) {
            //  this._secCounter++;
            for (int i = 0; i < base.MaximumIterations; i++) {
                base._sample = TargetSelection.GetImageSample(ref this._backProjection, base.PreviousArea);
                data = DistributionCalculator.GetDistributionInformations(base._sample);
                centroid = DistributionCalculator.RecalculateCentroidToArea(data.CentroidPosition, base.PreviousArea);
                base.CurrentArea = new AdaptiveTargetArea(base.PreviousArea);
                base.CurrentArea.OffSet(centroid.X, centroid.Y);
                if (TargetArea.CheckBounds(base.CurrentArea, ref bmp))
                    base.CurrentArea = base.PreviousArea;

                if (base.CurrentArea.Center.Equals(base.PreviousArea.Center) || i == base.MaximumIterations) {
                    iterations = i + 1;
                    //if (base.CurrentArea is IAdaptiveArea) {
                    //    (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.DistributionSize);
                    //}
                    //this._isReady = this.CheckAreas(this.CurrentArea, this.PreviousArea);
                    //base.PreviousArea = base.CurrentArea;
                    //base.m_sample = TargetSelection.GetImageSample(ref bmp, base.PreviousArea);
                    //base.HistogramGenerator.GenerateHistogram(SourceType.Target, ref base.m_sample, base.PreviousArea);


                    break;
                }
                base.PreviousArea = base.CurrentArea;
            }
            // }

            base.PreviousArea = base.CurrentArea;
            if (base.CurrentArea is IAdaptiveArea) {
                if (base.IsSimpleSizeCalculation) {
                    (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.Size);
                } else {
                    (base.CurrentArea as IAdaptiveArea).AdaptObjectArea(data.DistributionSize);
                }
            }
            //vypocitani histogramu z oblasti cile
            if (base.HistogramGenerator.GeneratorType != HistogramType.Weighted)
                base._sample = TargetSelection.GetImageSample(ref bmp, base.PreviousArea);
            else
                base._sample = TargetSelection.GetImageSample(ref bmp, base.PreviousArea.SearchingArea);
            base.HistogramGenerator.GenerateHistogram(SourceType.Target, ref base._sample, base.PreviousArea);
            this.OnTargetFound(new CAMShiftEventArgs(bmp, this._backProjection, null, base.CurrentArea,
                base.HistogramGenerator.CandidateHistogram, iterations, base.StopWatch.ElapsedMilliseconds, null));
            base.StopWatch.Reset();
        }
        private bool CheckAreas(TargetArea area1, TargetArea area2)
        {
            int difWidth = area1.ObjectArea.Width - area2.ObjectArea.Width;
            int difHeight = area1.ObjectArea.Height - area2.ObjectArea.Height;
            difWidth = difWidth < 0 ? difWidth * (-1) : difWidth;
            difHeight = difHeight < 0 ? difHeight * (-1) : difHeight;
            return (difWidth == 0 && difHeight == 0) || (difWidth < 2 && difHeight < 2);

        }
        #endregion
    }
}
