﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TrackerGraphic;
using RGBHistogramGenerator;
using System.Drawing;

namespace Tracker
{
    /// <summary>
    /// Trida slouzi jako argument udalosti, vyvolane pri Trackovani, umoznuje prenaset udaje o
    /// TargetArea, histogramu kandidata, batacharyy koeficientu a poctu meanshift iteraci
    /// </summary>
    public class TrackerEventArgs : EventArgs
    {
        #region Fields
        /// <summary>
        /// targetArea
        /// </summary>
        private TargetArea _targetArea;
        /// <summary>
        /// candidateHistogram
        /// </summary>
        private BaseHistogram _candidateHistogram;
        /// <summary>
        /// batacharry koeficient
        /// </summary>
        private double _batacharyyCoef;
        /// <summary>
        /// mean shift iterations
        /// </summary>
        private int _meanshiftIterations;
        /// <summary>
        /// Analyzovana bitmapa
        /// </summary>
        private Bitmap _calculatedBitmap;
        /// <summary>
        /// uplynuly cas vypoctu
        /// </summary>
        private long  _computationTime;
        #endregion //end fields

        #region Properties
        /// <summary>
        /// Get Target Area
        /// </summary>
        public TargetArea TargetArea
        {
            get
            {
                return this._targetArea;
            }
        }
        /// <summary>
        /// Get Histogram kandidata
        /// </summary>
        public BaseHistogram CandidateHistogram
        {
            get
            {
                return this._candidateHistogram;
            }
        }
        /// <summary>
        /// Get Batacharyy koeficient
        /// </summary>
        public double BatacharyyCoef
        {
            get
            {
                return this._batacharyyCoef;
            }
        }
        /// <summary>
        /// Get pocet mean - shift iteraci
        /// </summary>
        public int MeanshiftIterations
        {
            get
            {
                return this._meanshiftIterations;
            }
        }
        /// <summary>
        /// Get Bitmapa s kterou se pracovalo
        /// </summary>
        public Bitmap CalculatedBitmap
        {
            get
            {
                return this._calculatedBitmap;
            }
        }
        /// <summary>
        /// Get celkový čas výpočtu
        /// </summary>
        public long ComputationTime
        {
            get
            {
                return this._computationTime;
            }
        }
        #endregion // end properties

        #region Constructors
        /// <summary>
        /// Konstruktor argumentu udalosti
        /// </summary>
        /// <param name="targetArea">Oblast cile</param>
        /// <param name="candidateHistogram">histogram kandidata</param>
        /// <param name="batacharyyCoef">batacharyy koeficient</param>
        /// <param name="meanshiftIteration"> pocet meanshift iteraci</param>
        public TrackerEventArgs(Bitmap calculatedBitmap, TargetArea targetArea, BaseHistogram candidateHistogram,
            double batacharyyCoef, int meanshiftIteration, long time)
        {
            this._targetArea = targetArea;
            this._candidateHistogram = candidateHistogram;
            this._batacharyyCoef = batacharyyCoef;
            this._meanshiftIterations = meanshiftIteration;
            this._calculatedBitmap = calculatedBitmap;
            this._computationTime = time;

        }
        #endregion //end constructors
    }
}
