﻿using System.Drawing;

namespace TrackerGraphic
{
    public class Area
    {
        private Rectangle _rectangle;
        private Point _center;
        private Point _offSetPoint;
        /// <summary>
        /// Get Horni levy roh
        /// </summary>
        /// <value>The upper left corner.</value>
        public Point UpperLeftCorner
        {
            get
            {
                return this._rectangle.Location;
            }
        }
        /// <summary>
        /// Get spodni pravy roh.
        /// </summary>
        /// <value>The bottom right corner.</value>
        public Point BottomRightCorner
        {
            get
            {
                return new Point(this._rectangle.Right, this._rectangle.Bottom);
            }
        }
        /// <summary>
        /// Get obdelnik jenz reprezentuje oblast
        /// </summary>
        /// <value>The area rectangle.</value>
        public Rectangle AreaRectangle
        {
            get
            {
                return this._rectangle;
            }
            set
            {
                this._rectangle = value;
            }
        }
        /// <summary>
        /// Get Stred oblasti
        /// </summary>
        /// <value>The center.</value>
        public Point Center
        {
            get
            {
                return new Point(this._rectangle.Location.X + (this._rectangle.Width / 2),
                    this._rectangle.Location.Y + (this._rectangle.Height / 2));
            }
            set
            {
                this._center = value;
            }
        }
        public Point OffSetPoint
        {
            get
            {
                return this._offSetPoint;
            }
            set
            {
                this._offSetPoint = value;
            }

        }
        /// <summary>
        /// Get Sirka oblasti
        /// </summary>
        /// <value>The width.</value>
        public int Width
        {
            get
            {
                return this._rectangle.Width;
            }
        }
        /// <summary>
        /// Get Vyska oblasti
        /// </summary>
        /// <value>The height.</value>
        public int Height
        {
            get
            {
                return this._rectangle.Height;
            }
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="Area"/> class.
        /// </summary>
        /// <param name="startPosition">The start position.</param>
        /// <param name="endPosition">The end position.</param>
        public Area(Point startPosition, Point endPosition)
        {
            this._rectangle.Location = startPosition;
            this._rectangle = new Rectangle(startPosition, this.GetSize(endPosition));
        }
        public Area(Rectangle rec)
        {
            this._rectangle = rec;
        }
        /// <summary>
        /// Gets the size.
        /// </summary>
        /// <param name="endPosition">The end position.</param>
        /// <returns></returns>
        private Size GetSize(Point endPosition)
        {
            int start = endPosition.X - this.UpperLeftCorner.X;
            int end = endPosition.Y - this.UpperLeftCorner.Y;
            return new Size(start, end);
        }
        /// <summary>
        /// Posunuti na predane souradnice.
        /// </summary>
        /// <param name="x">The x.</param>
        /// <param name="y">The y.</param>
        public void OffSet(int x, int y)
        {
            this._rectangle.Offset(x, y);
        }
        /// <summary>
        /// Posunuti na predane souradnice podle bodu.
        /// </summary>
        /// <param name="newLocation">The new location.</param>
        public void OffSet(Point newLocation)
        {
            this.OffSet(newLocation.X, newLocation.Y);
        }
        /// <summary>
        /// Rozhodne zda bod na predanych souradnicich je zastoupen uvnitr oblasti
        /// </summary>
        /// <param name="x">souradnice x</param>
        /// <param name="y">souradnice y</param>
        /// <returns>
        /// 	<c>true</c> if [contains] [the specified x]; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(int x, int y)
        {
            return this._rectangle.Contains(x, y);
        }
        /// <summary>
        /// Roztahne oblast na pozadovanou vysku a sirku
        /// </summary>
        /// <param name="width">The width.</param>
        /// <param name="height">The height.</param>
        public void Inflate(int width, int height)
        {
            this._rectangle.Inflate(width, height);
        }
    }
}
